"""
Mapping of light levels to lux measurement values for SwitchBot Hub 3.

Source: After-sales consultation, line chart data provided by switchbot developers
"""

LIGHT_INTENSITY_MAP = {
    1: 0,
    2: 50,
    3: 90,
    4: 205,
    5: 317,
    6: 510,
    7: 610,
    8: 707,
    9: 801,
    10: 1023,
}
